#ifndef __RINTERFACE_H__
#define __RINTERFACE_H__

#include <qglobal.h>
#include <Qt3Support/q3socket.h>
#include <Qt3Support/q3process.h>
#include <qobject.h>
#include <Qt3Support/q3progressdialog.h>
#include <Qt3Support/q3progressbar.h>

#include <iostream>

using namespace std;

class Rinterface : public QObject {
  Q_OBJECT

public:
  // potential return statuses. For some reason qt apparently is not too
  // fond of exceptions; that is why I'm using all these enums.
  enum connectionStatus { OK, SOCKETERROR, CONNECTIONERROR, NOTSTARTED };

  // constructor
  Rinterface(bool startServer=true);
  // destructor
  ~Rinterface();
  // open a pipe to R
  connectionStatus establishConnection();
  // send an R command to R
  void sendCommand(QString command, bool progressDialog=false,
		   QString progressString = "");
  // send a message from R to the socket. 
  void sendRSocketMessage(QString message);
  // load an R data file into the R session
  bool loadDataFile(QString datafile);
  // return the current connection status
  connectionStatus getConnectionStatus() { return status; };
  //QString getSocketContent();
  //bool clearSocketContent();
public slots:
  // socket communication slots
  void socketHostFound() { cout << "Host found" << endl; };
  void socketErrorFound(int errorNum);
  void socketMessage();
  void socketConnected();
  void progressStarted(QString);
  void progressFinished();

  // process communication slots
  void processErrorMessage();
signals:
  void newSocketData( QString );
  void connected();
  void startProgressDialog(QString);
  void stopProgressDialog();
  
private:
  connectionStatus status;
  bool dataLoaded;
  bool socketEstablished;
  Q3Process *pipe;
  Q3Socket *socket;
  Q3ProgressDialog *progress;
  Q3ProgressBar *progressBar;
  int socketErrorCount;
  int maxSocketErrors;
  int socketPort;
  QString socketHost;
  void startSocket();
};

#endif // __RINTERFACE_H__

