/**********************
Displays a bicpl obj file using the open inventor toolkit

Still quite obviously primitive in its functionality ...
Written by Jason Lerch and Steve Robbins
**********************/

#define QT_CLEAN_NAMESPACE 1

#include <ctype.h>
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <string>

#include <qstring.h>
#include <qfile.h>

#include <Inventor/SbBasic.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoTexture2Transform.h>
#include <Inventor/nodes/SoTextureCoordinate2.h>
#include <Inventor/nodes/SoTextureCoordinateBinding.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/actions/SoRayPickAction.h>

#include <Inventor/SoPickedPoint.h>

#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/events/SoMouseButtonEvent.h>

// this has to come after the inventor nodes and before the Qt includes

// #include <bicInventor.h>

//int filename_extension_matches( char *, char *);

#include <ParseArgv.h>



//#include "pointInfoWindow.h"
#include "callbacks.h"
#include "brainApp.h"
#include "brainEnvironment.h"

#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>


// the variables to hold the arguments
char   *textureFile     = NULL;
int    textureColumnNumber     = 0;
char   *textureColumnName = NULL;
int    textureInvert     = 0;
int    showColourBar = 1;
int    Verbose = 0;

// the argument parsing table

ArgvInfo argTable[] = {
  { "-verbose", ARGV_CONSTANT, (char *) 1, (char *) &Verbose,
    "verbose mode." },
  { "-texture", ARGV_STRING, (char *)0, (char *) &textureFile,
    "Use this texture file for the colour coding." },
  {"-no_colour_bar", ARGV_CONSTANT, (char *) 0, (char *) &showColourBar,
   "Disable colour bar - fixes bug on some machines." },

  { "-column_number", ARGV_INT, (char *)0, (char *) &textureColumnNumber,
    "Column of texture file to load." },
  { "-column_name", ARGV_STRING, (char *)0, (char *) &textureColumnName,
    "Name of column of texture to load." },
  { "-invert", ARGV_CONSTANT, (char *) 1, (char *) &textureInvert,
    "Invert the vertex data." },

  {NULL, ARGV_END, NULL, NULL, NULL }
};

int main(int argc, char **argv) {

  using namespace std;
  bool colourBar = true;

  // parse the argument
  if (ParseArgv(&argc, argv, argTable, 0)) {
    cerr << "USAGE: brain-view [options] brain.obj [vertex_file.txt]" << endl;
    exit(1);
  }

  // disable colour bar if that option was chosen
  if (showColourBar == 0) {
    colourBar = false;
  }

  // initialise the widgets and application code. The object window
  // will be the top-level window, the volume viewer will be a 
  // child window
  brainApp *app = new brainApp( argc, argv, colourBar, (bool)Verbose );

  // check if a texture was specified. If not, load the default one.
  if ( textureFile == NULL ) {
    textureFile = (char *) app->environment->textureFile->latin1();
  }
  app->setTextureImage( textureFile );

  // collect the remaining arguments. Two types are acknowledged:
  // files ending with .txt or .verstats, where each line corresponds
  // to some floating point number about each vertex, or .obj files,
  // which contain geometry.

  bool loadedObj = false;

  for (int i=1; i < argc; i++) {
    QString argument(argv[i]);
    if ( argument.endsWith("obj") ) {
      // OBJ file
      app->setOBJFile( argv[i] );
      loadedObj = true;
    }
  
    else if ( argument.endsWith("txt") ||
             argument.endsWith("vertstats")) {
     // load a texture
     app->setTextureFile( argv[i] );
     
     if (textureColumnName != NULL) {
       app->setTextureCoordinates( textureInvert, -1, textureColumnName);
     }
     else {
       app->setTextureCoordinates( textureInvert, textureColumnNumber);
     }
   }
  }

  // allow for a default obj file to be loaded to avoid the need of
  // specifying two files on the command line. This is especially
  // useful for OS X. The default file should be a link in the home
  // directory.
  if (loadedObj == false) {
    QString objFile;
    char *homedir = getenv("HOME");
    objFile = homedir;
    objFile.append("/.brain-view-default.obj");
    if (QFile::exists( objFile )) {
      app->setOBJFile( objFile.latin1() );
    }
  }




  if( Verbose ) cout << "before creation of scene graph" << endl;

  // create the scene graph
  app->createSceneGraph();

  // register the callback
  app->objViewer->setEventCallback( viewerOverrideCB, app );

  if( Verbose ) cout << "after callback registration" << endl;
   // Make myCamera see everything.
   app->camera->viewAll(app->textureInfo, 
			app->textureViewer->getViewportRegion());

   if( Verbose ) cout << "after cameras" << endl;

  // associate scene graphs with renders
   //app->objViewer->setSceneGraph(app->rootObj);

  if( Verbose ) cout << "after setting of scene graph" << endl;

  app->textureViewer->setSceneGraph(app->textureInfo);
  // add a mouse callback
  app->mouseEventCB->addEventCallback(SoMouseButtonEvent::getClassTypeId(),
				      mouseButtonEventCB, 
				      app);

  if( Verbose ) cout << "before showing of viewers" << endl;
  // and show the viewers
  app->objViewer->show();
  app->textureViewer->show();
  app->textureWindow->show();
  SoQt::show( app->textureWindow );

  app->mainWindow->show();
  app->updateRangeWidgets();


  // and loop forever
  if( Verbose ) cout << "Just before loop" << endl;
  SoQt::mainLoop();

  return 0;
}




