#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/nodes/SoSeparator.h>


#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoExtSelection.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoPointSet.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoPolygonOffset.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/manips/SoTrackballManip.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/sensors/SoIdleSensor.h>
#include <assert.h>
#include <Inventor/SoInput.h>
#include <Inventor/SoOutput.h>
#include <Inventor/actions/SoBoxHighlightRenderAction.h>
#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/SbBSPTree.h>
#include <Inventor/details/SoFaceDetail.h>
#include <Inventor/details/SoPointDetail.h>

#include <string.h>
#include <iostream>

#include "brainApp.h"

// JPL: most of this code is borrowed from the extselection example
// inside SoGuiExamples from the Coin folks. Including the snarky comment
// below.


// This is not how it should be done, since triangles might be
// selected multiple times. We just wrote this piece of crap for demo
// purposes.

SbBSPTree mybsptree;
SbList <int32_t> myidxlist;
SbBSPTree mypointbsptree;

SoCoordinate3 * mycoords = NULL;
SoIndexedLineSet * mylineset = NULL;
SoCoordinate3 * mypointcoords = NULL;
SoPointSet * mypointset = NULL;

int trianglecounter;
int linecounter;
int pointcounter;

void
start_callback(void * userdata, SoSelection * sel)
{
  trianglecounter = 0;
  linecounter = 0;
  if (sel->policy.getValue() == SoSelection::SINGLE ||
      (sel->policy.getValue() == SoSelection::SHIFT && !((SoExtSelection*)sel)->wasShiftDown())) {
    mybsptree.clear();
    mypointbsptree.clear();
    myidxlist.truncate(0);

    // This line is very important or the 'selection-lines' will be selected during the next selection as
    // a part of the original object.
    mylineset->coordIndex.setNum(0);
  }
}

void
finish_callback(void * userdata, SoSelection * sel)
{
  int n = mybsptree.numPoints();
  mycoords->point.setNum(n);
  memcpy(mycoords->point.startEditing(), mybsptree.getPointsArrayPtr(), n*sizeof(SbVec3f));
  mycoords->point.finishEditing();

  n = myidxlist.getLength();
  mylineset->coordIndex.setNum(n);
  memcpy(mylineset->coordIndex.startEditing(), myidxlist.getArrayPtr(), n*sizeof(int32_t));
  mylineset->coordIndex.finishEditing();

  n = mypointbsptree.numPoints();
  mypointcoords->point.setNum(n);
  memcpy(mypointcoords->point.startEditing(), mypointbsptree.getPointsArrayPtr(), n*sizeof(SbVec3f));
  mypointcoords->point.finishEditing();

 //  std::cout << mybsptree.numPoints() << std::endl;

}

SbBool 
lineCB(void * userdata, 
       SoCallbackAction * action, 
       const SoPrimitiveVertex * v1, 
       const SoPrimitiveVertex * v2)
{
  const SbMatrix & mm = action->getModelMatrix();

  ++linecounter;

  SbVec3f vec1 = v1->getPoint();
  SbVec3f vec2 = v2->getPoint();
  
  mm.multVecMatrix(v1->getPoint(),vec1);
  mm.multVecMatrix(v2->getPoint(),vec2);

  myidxlist.append(mybsptree.addPoint(vec1)); // Ugly hack. A 'line' must have 3 points...
  myidxlist.append(mybsptree.addPoint(vec2));
  myidxlist.append(mybsptree.addPoint(vec1));
 
  myidxlist.append(-1);

  return FALSE;
}

SbBool 
pointCB(void * userdata, 
        SoCallbackAction * action, 
        const SoPrimitiveVertex * v1)
{
  const SbMatrix & mm = action->getModelMatrix();

  ++pointcounter;

  SbVec3f vec1 = v1->getPoint();

  mm.multVecMatrix(v1->getPoint(),vec1);

  mypointbsptree.addPoint(vec1);

  return FALSE;
}

SbBool 
triangleCB(void * userdata, 
           SoCallbackAction * action, 
           const SoPrimitiveVertex * v1, 
           const SoPrimitiveVertex * v2, 
           const SoPrimitiveVertex * v3)
{

  brainApp *app = (brainApp *) userdata;
  // FIXME: need to test if face has already been added
  const SbMatrix & mm = action->getModelMatrix();

  SbVec3f op;
  SbVec3f wp;

  ++trianglecounter;

  const SoDetail * detail = v1->getDetail();
  // if not face, select entire object
  if(!detail || !detail->isOfType(SoFaceDetail::getClassTypeId())){
    return TRUE;
  }

  const SoFaceDetail * fd = (const SoFaceDetail*) detail;
  assert(fd->isOfType(SoFaceDetail::getClassTypeId()));
  assert(fd);
  const SoPointDetail * pd;

  // along with the standard extselection behaviour have it colour
  // in the vertices.
  SbVec2f *vertValues = app->paintingTexCoord->point.startEditing();
  for (int i = 0; i <= fd->getNumPoints(); i++) {
    pd = fd->getPoint(i%fd->getNumPoints());
    //std::cout << "COORD: " << pd->getCoordinateIndex() << std::endl;
    vertValues[pd->getCoordinateIndex()].setValue(0, app->paintNumber);
    
    assert(pd);
    assert(pd->getCoordinateIndex() < action->getNumCoordinates());
    op = action->getCoordinate3(pd->getCoordinateIndex());
    mm.multVecMatrix(op, wp);
    myidxlist.append(mybsptree.addPoint(wp));
  }
  app->paintingTexCoord->point.finishEditing();
  myidxlist.append(-1);
  return FALSE;
}

SoSeparator * 
make_line_sg(void)
{
  SoSeparator * sep = new SoSeparator;
  SoLightModel * lm = new SoLightModel;
  lm->model = SoLightModel::BASE_COLOR;
  sep->addChild(lm);
  SoBaseColor * col = new SoBaseColor;
  col->rgb = SbColor(1.0f, 0.0f, 0.0f);
  sep->addChild(col);
  SoDrawStyle * ds = new SoDrawStyle;
  ds->lineWidth = 2;
  ds->pointSize = 4;
  sep->addChild(ds);

  SoSeparator *linesep = new SoSeparator;
  sep->addChild(linesep);
  mycoords = new SoCoordinate3;
  mylineset = new SoIndexedLineSet;
  linesep->addChild(mycoords);
  linesep->addChild(mylineset);
  SoSeparator *pointsep = new SoSeparator;
  sep->addChild(pointsep);
  mypointcoords = new SoCoordinate3;
  mypointcoords->point.setNum(0);
  mypointset = new SoPointSet;
  pointsep->addChild(mypointcoords);
  pointsep->addChild(mypointset);
  return sep;
}

// Print a quick instructions notice on stdout.
void
show_instructions( void )
{
  (void)fprintf(stdout, 
                "\nThis example program demonstrates the use of the SoExtSelection node type.\n"
                "\nQuick instructions:\n\n"
                "  * draw a polygon by selecting vertices with the left mouse button\n"
                "  * terminate polygon either by selecting the same point twice,\n"
                "    or by clicking the right mouse button.\n"
                "  * hold SHIFT to avoid clearing selection when selecting new triangles\n"
                "  * hit ESC to toggle back and forth to view mode\n\n"
                "  * Toggle keys:\n"
#if HAVE_SOEXTSELECTION_LASSOMODE
                "     'v' / 'a' = VISIBLE_SHAPES / ALL_SHAPES lasso-mode (default: ALL_SHAPES)\n"
#endif // HAVE_SOEXTSELECTION_LASSOMODE
                "     'f' / 'p' / '1' / '2' = FULL / PART / FULL_BBOX / PART_BBOX lasso-policy (default: PART) \n"
                "     'l' / 'r' / 'n' = LASSO / RECTANGLE / NOLASSO lasso type (default: LASSO)\n"
                "     's' / 't' / 'h' = SINGLE / TOGGLE / SHIFT selection policy (default: SHIFT)\n"
                "\n");
}

void
event_cb(void * userdata, SoEventCallback * node)
{

  brainApp *app = (brainApp *) userdata;
  const SoEvent * event = node->getEvent();

#if HAVE_SOEXTSELECTION_LASSOMODE
  if (SO_KEY_PRESS_EVENT(event, V)) {
    app->selection->lassoMode = SoExtSelection::VISIBLE_SHAPES;
    fprintf(stdout,"VISIBLE_SHAPES mode\n");
  }
  if (SO_KEY_PRESS_EVENT(event, A)) {
    app->selection->lassoMode = SoExtSelection::ALL_SHAPES;
    fprintf(stdout,"ALL_SHAPES mode\n");
  }
#endif // HAVE_SOEXTSELECTION_LASSOMODE
  if (SO_KEY_PRESS_EVENT(event, F)) {
    app->selection->lassoPolicy = SoExtSelection::FULL;
    fprintf(stdout,"FULL mode\n");
  }
  if (SO_KEY_PRESS_EVENT(event, P)) {
    app->selection->lassoPolicy = SoExtSelection::PART;
    fprintf(stdout,"PART mode\n");
  }
  if (SO_KEY_PRESS_EVENT(event, NUMBER_1)) {
    app->selection->lassoPolicy = SoExtSelection::FULL_BBOX;
    fprintf(stdout,"FULL_BBOX mode\n");
  }
  if (SO_KEY_PRESS_EVENT(event, NUMBER_2)) {
    app->selection->lassoPolicy = SoExtSelection::PART_BBOX;
    fprintf(stdout,"PART_BBOX mode\n");
  }
  if (SO_KEY_PRESS_EVENT(event, L)) {
    app->selection->lassoType = SoExtSelection::LASSO;
    fprintf(stdout,"Lasso mode\n");
  }
  if (SO_KEY_PRESS_EVENT(event, R)) {
    app->selection->lassoType = SoExtSelection::RECTANGLE;
    fprintf(stdout,"Rectangle mode\n");
  }
  if (SO_KEY_PRESS_EVENT(event, N)) {
    app->selection->lassoType = SoExtSelection::NOLASSO;
    fprintf(stdout,"NoLasso mode\n");
  }
  if (SO_KEY_PRESS_EVENT(event, S)) {
    app->selection->policy = SoSelection::SINGLE;
    fprintf(stdout,"Single selection\n");
  }
  if (SO_KEY_PRESS_EVENT(event, H)) {
    app->selection->policy = SoSelection::SHIFT;
    fprintf(stdout,"Shift selection\n");
  }
  if (SO_KEY_PRESS_EVENT(event, T)) {
    app->selection->policy = SoSelection::TOGGLE;
    // FIXME: should write up the necessary code in this example to
    // make toggle work for polygon/line/point primitive sets
    // aswell. 20020808 mortene.
    fprintf(stdout,"Toggle selection (note: only works for full shapes)\n");
  }
}


/*
int add_extselection(SoSeparator *root, SoQtExaminerViewer *viewer) {
  

  SoPolygonOffset * po = new SoPolygonOffset;
  po->styles = SoPolygonOffset::FILLED;
  po->factor = 1.0f;
  po->units = 1.0f;
  po->on = TRUE;
  root->insertChild(po, 0);



  selection = new SoExtSelection;
  selection->ref();
  selection->lassoPolicy = SoExtSelection::PART;
  selection->policy = SoSelection::SHIFT;
  selection->lassoType = SoExtSelection::LASSO;
  // #if HAVE_SOEXTSELECTION_LASSOMODE
  selection->lassoMode = SoExtSelection::VISIBLE_SHAPES;
  // #endif // HAVE_SOEXTSELECTION_LASSOMODE

  selection->setLassoColor(SbColor(0.9f,0.7f,0.2f));
  selection->setLassoWidth(3);
  selection->setOverlayLassoPattern(0xf0f0);  // 0011 0011 0011 0011
  selection->animateOverlayLasso(FALSE);
  selection->addStartCallback(start_callback);
  selection->addFinishCallback(finish_callback);
  selection->setTriangleFilterCallback(triangleCB, NULL);
  selection->setLineSegmentFilterCallback(lineCB, NULL);
  selection->setPointFilterCallback(pointCB, NULL);

  
  SoEventCallback * cb = new SoEventCallback;
  cb->addEventCallback(SoKeyboardEvent::getClassTypeId(),
                       event_cb, NULL);
  root->insertChild(cb, 0);
  
  selection->addChild(root);
  

  SoSeparator * newroot = new SoSeparator;
  newroot->ref();
  newroot->addChild(selection);
  SoSeparator * overlaylines = make_line_sg();
  newroot->addChild(overlaylines);

   
  viewer->setSceneGraph(newroot);
  //viewer->setSceneGraph(root);
  viewer->setTransparencyType(SoGLRenderAction::SORTED_OBJECT_BLEND);
  viewer->setGLRenderAction(new SoBoxHighlightRenderAction(SbViewportRegion()));
  viewer->setViewing(FALSE);
   

  SoSearchAction search;
  search.setType(SoExtSelection::getClassTypeId());
  search.setFind(SoSearchAction::TYPE);
  search.setInterest(SoSearchAction::ALL);
  search.apply(root);
  SoPathList & pathlist2 = search.getPaths();


  if(pathlist2.getLength() > 0) {
    (void) fprintf(stderr,
                   "ERROR: You cannot have two ExtSelection nodes beneath each other\n"
                   "in the same scenegraph!\n\n");
    return 0;
  }

  // Just check if we can draw a sensible set of overlay lines --
  // multiple cameras makes this very hard, so it's not supported in
  // this example program.

  search.setType(SoCamera::getClassTypeId());
  search.setFind(SoSearchAction::TYPE);
  search.setInterest(SoSearchAction::ALL);
  search.apply(root);
  SoPathList & pathlist = search.getPaths();
  if(pathlist.getLength() > 1) {
    (void) fprintf(stderr,
                   "WARNING: there is more than 1 camera in the loaded\n"
                   "scenegraph -- overlay lines for selection might come\n"
                   "out wrong!\n");

  }


  // Make sure the overlay lines we render are within the same camera
  // scope as the actual geometry from the input file.
  //overlaylines->insertChild(ex1->getCamera(), 0);
  
  return(0);
} 


*/
