#include <Qt3Support/q3hbox.h>
#include <Qt3Support/q3vbox.h>
#include <qlineedit.h>
#include <qlabel.h>
//#include <qapplication.h>

#include "pointInfoWindow.h"

coordinateInfoSet::coordinateInfoSet( QWidget *parent, const char *name )
                 : Q3HBox( parent, name ) {
  
  // some geometry management:
  //  this->setMaximumWidth( 200 );

  // initialise the widgets
  this->xLabel = new QLabel( "X:", this );
  this->xCoord = new QLineEdit( "0", this );
  this->yLabel = new QLabel( "Y:", this );
  this->yCoord = new QLineEdit( "0", this );
  this->zLabel = new QLabel( "Z:", this );
  this->zCoord = new QLineEdit( "0", this );
  this->indexLabel = new QLabel( "Index:", this );
  this->indexValue = new QLineEdit( "0", this );

  // connect some signals and slots
  connect(this, SIGNAL(xCoordChanged(const QString &)),
          this->xCoord, SLOT(setText(const QString &)));
  connect(this, SIGNAL(yCoordChanged(const QString &)),
          this->yCoord, SLOT(setText(const QString &)));
  connect(this, SIGNAL(zCoordChanged(const QString &)),
          this->zCoord, SLOT(setText(const QString &)));
  connect(this, SIGNAL(indexChanged(const QString &)),
          this->indexValue, SLOT(setText(const QString &)));
  connect(indexValue, SIGNAL(returnPressed()),
          this, SLOT(indexReturnPressed()));
}

void coordinateInfoSet::setCoordinates( float xCoord, float yCoord, 
                                        float zCoord, int index ){
  QString str;
  emit xCoordChanged( str.sprintf("%4.3f", xCoord) );
  emit yCoordChanged( str.sprintf("%4.3f", yCoord) );
  emit zCoordChanged( str.sprintf("%4.3f", zCoord) );
  emit indexChanged( str.sprintf("%i", index) );
}

void coordinateInfoSet::indexReturnPressed() {
  int currentIndex = indexValue->text().toInt();
  emit newIndex( currentIndex );
}

textureInfoSet::textureInfoSet( QWidget *parent, const char *name ) 
              : Q3HBox( parent, name ) {
  
  //  this->setMaximumWidth( 200 );

  // initialise the widgets
  this->absoluteLabel = new QLabel( "Texture:", this );
  this->absoluteTexture = new QLineEdit( "0", this );
  this->adjustedLabel = new QLabel( "Value:", this );
  this->adjustedTexture = new QLineEdit( "0", this );

  // connect signals and slots
  connect(this, SIGNAL(absoluteTextureChanged(const QString &)),
          this->absoluteTexture, SLOT(setText(const QString &)));
  connect(this, SIGNAL(adjustedTextureChanged(const QString &)),
          this->adjustedTexture, SLOT(setText(const QString &)));

  
}

void textureInfoSet::setTextureData( float absoluteTexture, float adjustedTexture ){
  QString str;
  emit absoluteTextureChanged( str.sprintf("%4.3f", absoluteTexture) );
  emit adjustedTextureChanged( str.sprintf("%4.3f", adjustedTexture) );
}

pointInfoWindow::pointInfoWindow( QWidget *parent, const char *name )
               : Q3VBox( parent, name ) {

  // initialise the widgets
  this->coordinates = new coordinateInfoSet( this );
  this->textureInfo = new textureInfoSet( this );
}

/*
  int main( int argc, char *argv[] ) {
  QApplication a( argc, argv );
  
  pointInfoWindow mine;
  a.setMainWidget( &mine );
  mine.show();
  return a.exec();
  }
*/
