#include <ctype.h>
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <string>

#include <Qt3Support/q3listbox.h>
#include <qlineedit.h>
#include <qcursor.h>
#include <qapplication.h>

#include "screenCapture.h"

using namespace std;

screenCapture::screenCapture( SoQtExaminerViewer *viewer,
                              QWidget* parent, const char* name,
                              bool modal, Qt::WindowFlags fl )
  : screenCaptureForm( parent, name, modal, fl ) {

  // any other initialisation code that might be necessary goes here
  this->counter = 0;
  this->viewer = viewer;

}

screenCapture::~screenCapture() { /* NOOP */ }

// a slot that is called when the capture button is pressed
void screenCapture::capture() {

  char filename[255];

  // Create a viewport to render the scene into.
  const SbViewportRegion &vp = this->viewer->getViewportRegion();
  
  // generate the filename from the counter, prefix, and file type.
  QString formatText = dataFormatBox->currentText().lower();
  // change from "postscript" to "ps"
  if (formatText == "postscript")
    formatText = "ps";

  QString prefix = filenamePrefixEdit->text();
  sprintf(filename, "%s_%03d.%s", prefix.latin1(), counter, 
          formatText.latin1());

  cout << "Dumping file ..." << filename << endl;    

  // increase the counter for the next screen capture
  counter++;

  // get and set the sizes and DPI
  int xSize = xSizeEdit->text().toInt();
  int ySize = ySizeEdit->text().toInt();
  float DPI = DPIEdit->text().toFloat();

  // create the viewport to render into
  SbViewportRegion vpr = viewer->getViewportRegion();
  vpr.setViewportPixels(0,0, xSize, ySize);
  vpr.setPixelsPerInch(DPI); // appears to have no effect

  // create the renderer and get the node to be rendered
  SoNode *root = viewer->getSceneManager()->getSceneGraph();
  SoOffscreenRenderer *renderer = new SoOffscreenRenderer(vpr);

  // do the rendering
  QApplication::setOverrideCursor( Qt::WaitCursor );
  renderer->setBackgroundColor( viewer->getBackgroundColor() );
  renderer->render(root);
  QApplication::restoreOverrideCursor();

  // write to file
  if ( dataFormatBox->currentText() == "RGB" )
    renderer->writeToRGB(filename);
  else if ( dataFormatBox->currentText() == "PostScript" )
    renderer->writeToPostScript(filename, SbVec2f(xSize, ySize));
}
